package djmil.cordacheckers.contracts;

import java.util.List;

import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.annotations.CordaSerializable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.ledger.utxo.Command;

public class GameBoardCommand implements Command {

    @CordaSerializable
    public static enum Type {
        MOVE,
        SURRENDER,
        REQUEST_DRAW,
        REQUEST_VICTORY,
        FINISH; // aka accept DRAW or VICTORY request
    }

    private final Type type;
    private final List<Integer> move; // [0] from, [1] to

    // Serialisation service requires a default constructor
    public GameBoardCommand() {
        this.type = null;
        this.move = null;
    }

    @ConstructorForDeserialization
    public GameBoardCommand(Type type, List<Integer> move) {
        this.type = type;
        this.move = move;
    }

    public GameBoardCommand(Type type) {
        if (type == Type.MOVE)
            throw new CordaRuntimeException (BAD_ACTIONMOVE_CONSTRUCTOR);

        this.type = type;
        this.move = null;
    }

    public GameBoardCommand(List<Integer> move) {
        this.type = Type.MOVE;
        this.move = move;
    }

    public Type getType() {
        return this.type;
    }

    public List<Integer> getMove() {
        return this.move;
    }

    static final String BAD_ACTIONMOVE_CONSTRUCTOR = "Bad constructor for Action.MOVE";
}
