package djmil.cordacheckers.contracts;

import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleCommand;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleInputState;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleOutputState;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleReferenceState;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.ledger.utxo.Command;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class GameBoardContract implements net.corda.v5.ledger.utxo.Contract {

    private final static Logger log = LoggerFactory.getLogger(GameBoardContract.class);

    @Override
    public void verify(UtxoLedgerTransaction trx) {
        log.info("GameBoardContract.verify() called");

        final Command command = getSingleCommand(trx, Command.class);

        if (command instanceof GameProposalCommand) {
            log.info("GameBoardContract.verify() as GameProposalCommand "+(GameProposalCommand)command);
        } else 
        if (command instanceof GameBoardCommand) {
            log.info("GameBoardContract.verify() as GameBoardCommand "+((GameBoardCommand)command).getType());
            switch (((GameBoardCommand)command).getType()) {
                case MOVE:
                break;
                case SURRENDER:
                break;
                case REQUEST_DRAW:
                break;
                case REQUEST_VICTORY:
                break;
                case FINISH:
                break;
            }
        } else {
            throw new CordaRuntimeException(UNKNOWN_COMMAND);
        }
    }

    @Suspendable
    public static GameProposalState getReferanceGameProposalState(UtxoLedgerTransaction trx) {
        final Command command = getSingleCommand(trx, Command.class);

        if (command instanceof GameProposalCommand && (GameProposalCommand)command == GameProposalCommand.ACCEPT) {
            return getSingleInputState(trx, GameProposalState.class);
        } else 
        if (command instanceof GameBoardCommand) {
            return getSingleReferenceState(trx, GameProposalState.class);
        }

        throw new IllegalStateException(NO_REFERANCE_GAMEPROPOSAL_STATE_FOR_TRXID +trx.getId());
    }
    
    private static void requireThat(boolean asserted, String errorMessage) {
        if(!asserted) {
            throw new CordaRuntimeException("Failed requirement: " + errorMessage);
        }
    }

    static final String REQUIRE_SINGLE_COMMAND = "Require a single command";
    static final String UNKNOWN_COMMAND = "Unsupported command";

    static final String SINGLE_STATE_EXPECTED = "Single state expected";
    static final String NO_REFERANCE_GAMEPROPOSAL_STATE_FOR_TRXID = "No reference GamePropsal state found for trx.id ";
}
