package djmil.cordacheckers.contracts;

import java.util.List;
import java.util.Optional;

import net.corda.v5.ledger.utxo.Command;
import net.corda.v5.ledger.utxo.ContractState;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class UtxoLedgerTransactionUtil {
    public static <T extends Command> T getSingleCommand(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return single(utxoTrx.getCommands(clazz), clazz);
    }

    public static <T extends ContractState> T getSingleReferenceState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return single(utxoTrx.getReferenceStates(clazz), clazz);
    }

    public static <T extends ContractState> T getSingleInputState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return single(utxoTrx.getInputStates(clazz), clazz);
    }

    public static <T extends ContractState> T getSingleOutputState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return single(utxoTrx.getOutputStates(clazz), clazz);
    }

    public static <T extends Command> Optional<T> getOptionalCommand(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return optional(utxoTrx.getCommands(clazz), clazz);
    }

    public static <T extends ContractState> Optional<T> getOptionalReferenceState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return optional(utxoTrx.getReferenceStates(clazz), clazz);
    }

    public static <T extends ContractState> Optional<T> getOptionalInputState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return optional(utxoTrx.getInputStates(clazz), clazz);
    }

    public static <T extends ContractState> Optional<T> getOptionalOutputState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return optional(utxoTrx.getOutputStates(clazz), clazz);
    }

    private static <T> Optional<T> optional(List<T> list, Class<T> clazz) {
        return list
            .stream()
            .reduce((a, b) -> {throw new IllegalStateException(MULTIPLE_INSTANCES_OF +clazz.getName());});
    }

    private static <T> T single(List<T> list, Class<T> clazz) {
        return optional(list, clazz)
            .orElseThrow( () -> new IllegalStateException(NO_INSTANCES_OF +clazz.getName()) );
    }

    private static String MULTIPLE_INSTANCES_OF = "Multiple instances of ";
    private static String NO_INSTANCES_OF = "No instances of ";
}
