package djmil.cordacheckers.gameboard;

import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.FlowResult;
import djmil.cordacheckers.contracts.GameBoardCommand;
import djmil.cordacheckers.contracts.GameBoardContract;
import djmil.cordacheckers.contracts.GameProposalCommand;
import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoTransactionBuilder;

import java.time.Duration;
import java.time.Instant;

public class CommandFlow implements ClientStartableFlow {

    private final static Logger log = LoggerFactory.getLogger(CommandFlow.class);

    @CordaInject
    public JsonMarshallingService jsonMarshallingService;

    @CordaInject
    public UtxoLedgerService utxoLedgerService;

    @CordaInject
    public FlowEngine flowEngine;

    @CordaInject
    public MemberLookup memberLookup;

    @Override
    @Suspendable
    public String call(ClientRequestBody requestBody) {
        try {
            final CommandFlowArgs args = requestBody.getRequestBodyAs(jsonMarshallingService, CommandFlowArgs.class);
            //final GameCommand command = args.getCommand();
log.info("GameBoardCommandFlow: findUnconsumedGameBoardState");
            final StateAndRef<GameBoardState> gbStateAndRef = findUnconsumedGameBoardState(args.getGameBoardUuid());

            // final UtxoSignedTransaction trx = prepareSignedTransaction(command, utxoGameProposal);

            // final SecureHash trxId = this.flowEngine
            //     .subFlow( new CommitSubFlow(trx, command.getRespondent(utxoGameProposal)) );

            // if (command == GameProposalCommand.ACCEPT) {
            //     GameBoardState newGb = (GameBoardState)trx.getOutputStateAndRefs().get(0).getState().getContractState();

            //     return new FlowResult(newGb.getId(), trxId) 
            //         .toJsonEncodedString(jsonMarshallingService);
            // }
   log.info("GameBoardCommandFlow: prepareGameBoardView");         
            GameBoardView gbView = prepareGameBoardView(gbStateAndRef);

            return new FlowResult(gbView )
                .toJsonEncodedString(jsonMarshallingService);
        } 
        catch (Exception e) {
            log.warn("GameProposalAction flow failed to process utxo request body " + requestBody + 
                        " because: " + e.getMessage());
            return new FlowResult(e).toJsonEncodedString(jsonMarshallingService);
        }
    }

    @Suspendable
    private StateAndRef<GameBoardState> findUnconsumedGameBoardState (UUID gameBoardUuid) {
        /*
         * Get list of all unconsumed aka 'active' GameProposalStates, then filter by UUID.
         * Note, this is an inefficient way to perform this operation if there are a large
         * number of 'active' GameProposals exists in storage.
         */
        return this.utxoLedgerService
            .findUnconsumedStatesByType(GameBoardState.class)
            .stream()
            .filter(sar -> sar.getState().getContractState().getId().equals(gameBoardUuid))
            .reduce((a, b) -> {throw new IllegalStateException("Multiple states: " +a +", " +b);})
            .get();
    }

    // @Suspendable
    // private UtxoSignedTransaction prepareSignedTransaction(
    //     GameProposalCommand command,
    //     StateAndRef<GameProposalState> utxoGameProposal
    // ) {
    //     UtxoTransactionBuilder trxBuilder = ledgerService.createTransactionBuilder()
    //         .setNotary(utxoGameProposal.getState().getNotaryName())
    //         .setTimeWindowBetween(Instant.now(), Instant.now().plusMillis(Duration.ofDays(1).toMillis()))
    //         .addInputState(utxoGameProposal.getRef())
    //         .addCommand(command)
    //         .addSignatories(utxoGameProposal.getState().getContractState().getParticipants());

    //     if (command == GameProposalCommand.ACCEPT) {
    //         trxBuilder = trxBuilder
    //             .addOutputState(new GameBoardState(utxoGameProposal));
    //             //A state cannot be both an input and a reference input in the same transaction
    //             //.addReferenceState(utxoGameProposal.getRef());
    //     }

    //     return trxBuilder.toSignedTransaction();
    // }

    @Suspendable
    private GameBoardView prepareGameBoardView(StateAndRef<GameBoardState> stateAndRef) {
        final MemberX500Name myName = memberLookup.myInfo().getName();

        final SecureHash trxId = stateAndRef.getRef().getTransactionId();

        final UtxoLedgerTransaction utxoGameBoard = utxoLedgerService
            .findLedgerTransaction(trxId);
log.info("GameBoardCommandFlow: createw gbView");
        GameBoardView gbView = new GameBoardView(myName, utxoGameBoard);

        gbView.previousCommand = new GameBoardCommand(GameBoardCommand.Type.SURRENDER);

        return gbView;
       // return new GameBoardView(myName, utxoGameBoard);
    }

}
