package djmil.cordacheckers.gameboard;

import java.util.Map;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameBoardCommand;
import djmil.cordacheckers.contracts.GameBoardContract;
import djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil;
import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.GameProposalState;
import djmil.cordacheckers.states.Piece;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;


// GameBoard from the player's point of view
public class GameBoardView {
    public final String opponentName;
    public final Piece.Color opponentColor;
    public final Boolean opponentMove;
    public final Map<Integer, Piece> board;
    public /*final*/ GameBoardCommand previousCommand;
    public final String message;
    public final UUID id;

    // Serialisation service requires a default constructor
    public GameBoardView() {
        this.opponentName = null;
        this.opponentColor = null;
        this.opponentMove = null;
        this.board = null;
        this.previousCommand = null;
        this.message = null;
        this.id = null;
    }

    // A view from a perspective of a concrete player, on a ledger transaction that has 
    // produced new GameBoardState
    public GameBoardView(MemberX500Name myName, UtxoLedgerTransaction utxoGameBoard) {

        final GameProposalState referanceGameProposal = GameBoardContract
            .getReferanceGameProposalState(utxoGameBoard);

        this.opponentName  = referanceGameProposal.getOpponentName(myName).getCommonName();
        this.opponentColor = referanceGameProposal.getOpponentColor(myName);

        final GameBoardState stateGameBoard = UtxoLedgerTransactionUtil
            .getSingleOutputState(utxoGameBoard, GameBoardState.class);

        this.opponentMove  = this.opponentColor == stateGameBoard.getMoveColor();
        this.board   = stateGameBoard.getBoard();
        this.message = stateGameBoard.getMessage();
        this.id      = stateGameBoard.getId();

        this.previousCommand = UtxoLedgerTransactionUtil
            .getOptionalCommand(utxoGameBoard, GameBoardCommand.class)
            .orElseGet(() -> null);
    }

}
